@@ -194,158 +194,161 @@ extern int yydebug;
     VAR_HTTP_QUERY_BUFFER_SIZE = 400,
     VAR_HTTP_RESPONSE_BUFFER_SIZE = 401,
     VAR_HTTP_NODELAY = 402,
-    VAR_STUB_FIRST = 403,
-    VAR_MINIMAL_RESPONSES = 404,
-    VAR_RRSET_ROUNDROBIN = 405,
-    VAR_MAX_UDP_SIZE = 406,
-    VAR_DELAY_CLOSE = 407,
-    VAR_UNBLOCK_LAN_ZONES = 408,
-    VAR_INSECURE_LAN_ZONES = 409,
-    VAR_INFRA_CACHE_MIN_RTT = 410,
-    VAR_DNS64_PREFIX = 411,
-    VAR_DNS64_SYNTHALL = 412,
-    VAR_DNS64_IGNORE_AAAA = 413,
-    VAR_DNSTAP = 414,
-    VAR_DNSTAP_ENABLE = 415,
-    VAR_DNSTAP_SOCKET_PATH = 416,
-    VAR_DNSTAP_IP = 417,
-    VAR_DNSTAP_TLS = 418,
-    VAR_DNSTAP_TLS_SERVER_NAME = 419,
-    VAR_DNSTAP_TLS_CERT_BUNDLE = 420,
-    VAR_DNSTAP_TLS_CLIENT_KEY_FILE = 421,
-    VAR_DNSTAP_TLS_CLIENT_CERT_FILE = 422,
-    VAR_DNSTAP_SEND_IDENTITY = 423,
-    VAR_DNSTAP_SEND_VERSION = 424,
-    VAR_DNSTAP_BIDIRECTIONAL = 425,
-    VAR_DNSTAP_IDENTITY = 426,
-    VAR_DNSTAP_VERSION = 427,
-    VAR_DNSTAP_LOG_RESOLVER_QUERY_MESSAGES = 428,
-    VAR_DNSTAP_LOG_RESOLVER_RESPONSE_MESSAGES = 429,
-    VAR_DNSTAP_LOG_CLIENT_QUERY_MESSAGES = 430,
-    VAR_DNSTAP_LOG_CLIENT_RESPONSE_MESSAGES = 431,
-    VAR_DNSTAP_LOG_FORWARDER_QUERY_MESSAGES = 432,
-    VAR_DNSTAP_LOG_FORWARDER_RESPONSE_MESSAGES = 433,
-    VAR_RESPONSE_IP_TAG = 434,
-    VAR_RESPONSE_IP = 435,
-    VAR_RESPONSE_IP_DATA = 436,
-    VAR_HARDEN_ALGO_DOWNGRADE = 437,
-    VAR_IP_TRANSPARENT = 438,
-    VAR_IP_DSCP = 439,
-    VAR_DISABLE_DNSSEC_LAME_CHECK = 440,
-    VAR_IP_RATELIMIT = 441,
-    VAR_IP_RATELIMIT_SLABS = 442,
-    VAR_IP_RATELIMIT_SIZE = 443,
-    VAR_RATELIMIT = 444,
-    VAR_RATELIMIT_SLABS = 445,
-    VAR_RATELIMIT_SIZE = 446,
-    VAR_RATELIMIT_FOR_DOMAIN = 447,
-    VAR_RATELIMIT_BELOW_DOMAIN = 448,
-    VAR_IP_RATELIMIT_FACTOR = 449,
-    VAR_RATELIMIT_FACTOR = 450,
-    VAR_SEND_CLIENT_SUBNET = 451,
-    VAR_CLIENT_SUBNET_ZONE = 452,
-    VAR_CLIENT_SUBNET_ALWAYS_FORWARD = 453,
-    VAR_CLIENT_SUBNET_OPCODE = 454,
-    VAR_MAX_CLIENT_SUBNET_IPV4 = 455,
-    VAR_MAX_CLIENT_SUBNET_IPV6 = 456,
-    VAR_MIN_CLIENT_SUBNET_IPV4 = 457,
-    VAR_MIN_CLIENT_SUBNET_IPV6 = 458,
-    VAR_MAX_ECS_TREE_SIZE_IPV4 = 459,
-    VAR_MAX_ECS_TREE_SIZE_IPV6 = 460,
-    VAR_CAPS_WHITELIST = 461,
-    VAR_CACHE_MAX_NEGATIVE_TTL = 462,
-    VAR_PERMIT_SMALL_HOLDDOWN = 463,
-    VAR_QNAME_MINIMISATION = 464,
-    VAR_QNAME_MINIMISATION_STRICT = 465,
-    VAR_IP_FREEBIND = 466,
-    VAR_DEFINE_TAG = 467,
-    VAR_LOCAL_ZONE_TAG = 468,
-    VAR_ACCESS_CONTROL_TAG = 469,
-    VAR_LOCAL_ZONE_OVERRIDE = 470,
-    VAR_ACCESS_CONTROL_TAG_ACTION = 471,
-    VAR_ACCESS_CONTROL_TAG_DATA = 472,
-    VAR_VIEW = 473,
-    VAR_ACCESS_CONTROL_VIEW = 474,
-    VAR_VIEW_FIRST = 475,
-    VAR_SERVE_EXPIRED = 476,
-    VAR_SERVE_EXPIRED_TTL = 477,
-    VAR_SERVE_EXPIRED_TTL_RESET = 478,
-    VAR_SERVE_EXPIRED_REPLY_TTL = 479,
-    VAR_SERVE_EXPIRED_CLIENT_TIMEOUT = 480,
-    VAR_FAKE_DSA = 481,
-    VAR_FAKE_SHA1 = 482,
-    VAR_LOG_IDENTITY = 483,
-    VAR_HIDE_TRUSTANCHOR = 484,
-    VAR_TRUST_ANCHOR_SIGNALING = 485,
-    VAR_AGGRESSIVE_NSEC = 486,
-    VAR_USE_SYSTEMD = 487,
-    VAR_SHM_ENABLE = 488,
-    VAR_SHM_KEY = 489,
-    VAR_ROOT_KEY_SENTINEL = 490,
-    VAR_DNSCRYPT = 491,
-    VAR_DNSCRYPT_ENABLE = 492,
-    VAR_DNSCRYPT_PORT = 493,
-    VAR_DNSCRYPT_PROVIDER = 494,
-    VAR_DNSCRYPT_SECRET_KEY = 495,
-    VAR_DNSCRYPT_PROVIDER_CERT = 496,
-    VAR_DNSCRYPT_PROVIDER_CERT_ROTATED = 497,
-    VAR_DNSCRYPT_SHARED_SECRET_CACHE_SIZE = 498,
-    VAR_DNSCRYPT_SHARED_SECRET_CACHE_SLABS = 499,
-    VAR_DNSCRYPT_NONCE_CACHE_SIZE = 500,
-    VAR_DNSCRYPT_NONCE_CACHE_SLABS = 501,
-    VAR_IPSECMOD_ENABLED = 502,
-    VAR_IPSECMOD_HOOK = 503,
-    VAR_IPSECMOD_IGNORE_BOGUS = 504,
-    VAR_IPSECMOD_MAX_TTL = 505,
-    VAR_IPSECMOD_WHITELIST = 506,
-    VAR_IPSECMOD_STRICT = 507,
-    VAR_CACHEDB = 508,
-    VAR_CACHEDB_BACKEND = 509,
-    VAR_CACHEDB_SECRETSEED = 510,
-    VAR_CACHEDB_REDISHOST = 511,
-    VAR_CACHEDB_REDISPORT = 512,
-    VAR_CACHEDB_REDISTIMEOUT = 513,
-    VAR_CACHEDB_REDISEXPIRERECORDS = 514,
-    VAR_UDP_UPSTREAM_WITHOUT_DOWNSTREAM = 515,
-    VAR_FOR_UPSTREAM = 516,
-    VAR_AUTH_ZONE = 517,
-    VAR_ZONEFILE = 518,
-    VAR_MASTER = 519,
-    VAR_URL = 520,
-    VAR_FOR_DOWNSTREAM = 521,
-    VAR_FALLBACK_ENABLED = 522,
-    VAR_TLS_ADDITIONAL_PORT = 523,
-    VAR_LOW_RTT = 524,
-    VAR_LOW_RTT_PERMIL = 525,
-    VAR_FAST_SERVER_PERMIL = 526,
-    VAR_FAST_SERVER_NUM = 527,
-    VAR_ALLOW_NOTIFY = 528,
-    VAR_TLS_WIN_CERT = 529,
-    VAR_TCP_CONNECTION_LIMIT = 530,
-    VAR_FORWARD_NO_CACHE = 531,
-    VAR_STUB_NO_CACHE = 532,
-    VAR_LOG_SERVFAIL = 533,
-    VAR_DENY_ANY = 534,
-    VAR_UNKNOWN_SERVER_TIME_LIMIT = 535,
-    VAR_LOG_TAG_QUERYREPLY = 536,
-    VAR_STREAM_WAIT_SIZE = 537,
-    VAR_TLS_CIPHERS = 538,
-    VAR_TLS_CIPHERSUITES = 539,
-    VAR_TLS_USE_SNI = 540,
-    VAR_IPSET = 541,
-    VAR_IPSET_NAME_V4 = 542,
-    VAR_IPSET_NAME_V6 = 543,
-    VAR_TLS_SESSION_TICKET_KEYS = 544,
-    VAR_RPZ = 545,
-    VAR_TAGS = 546,
-    VAR_RPZ_ACTION_OVERRIDE = 547,
-    VAR_RPZ_CNAME_OVERRIDE = 548,
-    VAR_RPZ_LOG = 549,
-    VAR_RPZ_LOG_NAME = 550,
-    VAR_DYNLIB = 551,
-    VAR_DYNLIB_FILE = 552,
-    VAR_EDNS_CLIENT_TAG = 553,
-    VAR_EDNS_CLIENT_TAG_OPCODE = 554
+    VAR_HTTP_NOTLS_DOWNSTREAM = 403,
+    VAR_STUB_FIRST = 404,
+    VAR_MINIMAL_RESPONSES = 405,
+    VAR_RRSET_ROUNDROBIN = 406,
+    VAR_MAX_UDP_SIZE = 407,
+    VAR_DELAY_CLOSE = 408,
+    VAR_UDP_CONNECT = 409,
+    VAR_UNBLOCK_LAN_ZONES = 410,
+    VAR_INSECURE_LAN_ZONES = 411,
+    VAR_INFRA_CACHE_MIN_RTT = 412,
+    VAR_INFRA_KEEP_PROBING = 413,
+    VAR_DNS64_PREFIX = 414,
+    VAR_DNS64_SYNTHALL = 415,
+    VAR_DNS64_IGNORE_AAAA = 416,
+    VAR_DNSTAP = 417,
+    VAR_DNSTAP_ENABLE = 418,
+    VAR_DNSTAP_SOCKET_PATH = 419,
+    VAR_DNSTAP_IP = 420,
+    VAR_DNSTAP_TLS = 421,
+    VAR_DNSTAP_TLS_SERVER_NAME = 422,
+    VAR_DNSTAP_TLS_CERT_BUNDLE = 423,
+    VAR_DNSTAP_TLS_CLIENT_KEY_FILE = 424,
+    VAR_DNSTAP_TLS_CLIENT_CERT_FILE = 425,
+    VAR_DNSTAP_SEND_IDENTITY = 426,
+    VAR_DNSTAP_SEND_VERSION = 427,
+    VAR_DNSTAP_BIDIRECTIONAL = 428,
+    VAR_DNSTAP_IDENTITY = 429,
+    VAR_DNSTAP_VERSION = 430,
+    VAR_DNSTAP_LOG_RESOLVER_QUERY_MESSAGES = 431,
+    VAR_DNSTAP_LOG_RESOLVER_RESPONSE_MESSAGES = 432,
+    VAR_DNSTAP_LOG_CLIENT_QUERY_MESSAGES = 433,
+    VAR_DNSTAP_LOG_CLIENT_RESPONSE_MESSAGES = 434,
+    VAR_DNSTAP_LOG_FORWARDER_QUERY_MESSAGES = 435,
+    VAR_DNSTAP_LOG_FORWARDER_RESPONSE_MESSAGES = 436,
+    VAR_RESPONSE_IP_TAG = 437,
+    VAR_RESPONSE_IP = 438,
+    VAR_RESPONSE_IP_DATA = 439,
+    VAR_HARDEN_ALGO_DOWNGRADE = 440,
+    VAR_IP_TRANSPARENT = 441,
+    VAR_IP_DSCP = 442,
+    VAR_DISABLE_DNSSEC_LAME_CHECK = 443,
+    VAR_IP_RATELIMIT = 444,
+    VAR_IP_RATELIMIT_SLABS = 445,
+    VAR_IP_RATELIMIT_SIZE = 446,
+    VAR_RATELIMIT = 447,
+    VAR_RATELIMIT_SLABS = 448,
+    VAR_RATELIMIT_SIZE = 449,
+    VAR_RATELIMIT_FOR_DOMAIN = 450,
+    VAR_RATELIMIT_BELOW_DOMAIN = 451,
+    VAR_IP_RATELIMIT_FACTOR = 452,
+    VAR_RATELIMIT_FACTOR = 453,
+    VAR_SEND_CLIENT_SUBNET = 454,
+    VAR_CLIENT_SUBNET_ZONE = 455,
+    VAR_CLIENT_SUBNET_ALWAYS_FORWARD = 456,
+    VAR_CLIENT_SUBNET_OPCODE = 457,
+    VAR_MAX_CLIENT_SUBNET_IPV4 = 458,
+    VAR_MAX_CLIENT_SUBNET_IPV6 = 459,
+    VAR_MIN_CLIENT_SUBNET_IPV4 = 460,
+    VAR_MIN_CLIENT_SUBNET_IPV6 = 461,
+    VAR_MAX_ECS_TREE_SIZE_IPV4 = 462,
+    VAR_MAX_ECS_TREE_SIZE_IPV6 = 463,
+    VAR_CAPS_WHITELIST = 464,
+    VAR_CACHE_MAX_NEGATIVE_TTL = 465,
+    VAR_PERMIT_SMALL_HOLDDOWN = 466,
+    VAR_QNAME_MINIMISATION = 467,
+    VAR_QNAME_MINIMISATION_STRICT = 468,
+    VAR_IP_FREEBIND = 469,
+    VAR_DEFINE_TAG = 470,
+    VAR_LOCAL_ZONE_TAG = 471,
+    VAR_ACCESS_CONTROL_TAG = 472,
+    VAR_LOCAL_ZONE_OVERRIDE = 473,
+    VAR_ACCESS_CONTROL_TAG_ACTION = 474,
+    VAR_ACCESS_CONTROL_TAG_DATA = 475,
+    VAR_VIEW = 476,
+    VAR_ACCESS_CONTROL_VIEW = 477,
+    VAR_VIEW_FIRST = 478,
+    VAR_SERVE_EXPIRED = 479,
+    VAR_SERVE_EXPIRED_TTL = 480,
+    VAR_SERVE_EXPIRED_TTL_RESET = 481,
+    VAR_SERVE_EXPIRED_REPLY_TTL = 482,
+    VAR_SERVE_EXPIRED_CLIENT_TIMEOUT = 483,
+    VAR_FAKE_DSA = 484,
+    VAR_FAKE_SHA1 = 485,
+    VAR_LOG_IDENTITY = 486,
+    VAR_HIDE_TRUSTANCHOR = 487,
+    VAR_TRUST_ANCHOR_SIGNALING = 488,
+    VAR_AGGRESSIVE_NSEC = 489,
+    VAR_USE_SYSTEMD = 490,
+    VAR_SHM_ENABLE = 491,
+    VAR_SHM_KEY = 492,
+    VAR_ROOT_KEY_SENTINEL = 493,
+    VAR_DNSCRYPT = 494,
+    VAR_DNSCRYPT_ENABLE = 495,
+    VAR_DNSCRYPT_PORT = 496,
+    VAR_DNSCRYPT_PROVIDER = 497,
+    VAR_DNSCRYPT_SECRET_KEY = 498,
+    VAR_DNSCRYPT_PROVIDER_CERT = 499,
+    VAR_DNSCRYPT_PROVIDER_CERT_ROTATED = 500,
+    VAR_DNSCRYPT_SHARED_SECRET_CACHE_SIZE = 501,
+    VAR_DNSCRYPT_SHARED_SECRET_CACHE_SLABS = 502,
+    VAR_DNSCRYPT_NONCE_CACHE_SIZE = 503,
+    VAR_DNSCRYPT_NONCE_CACHE_SLABS = 504,
+    VAR_IPSECMOD_ENABLED = 505,
+    VAR_IPSECMOD_HOOK = 506,
+    VAR_IPSECMOD_IGNORE_BOGUS = 507,
+    VAR_IPSECMOD_MAX_TTL = 508,
+    VAR_IPSECMOD_WHITELIST = 509,
+    VAR_IPSECMOD_STRICT = 510,
+    VAR_CACHEDB = 511,
+    VAR_CACHEDB_BACKEND = 512,
+    VAR_CACHEDB_SECRETSEED = 513,
+    VAR_CACHEDB_REDISHOST = 514,
+    VAR_CACHEDB_REDISPORT = 515,
+    VAR_CACHEDB_REDISTIMEOUT = 516,
+    VAR_CACHEDB_REDISEXPIRERECORDS = 517,
+    VAR_UDP_UPSTREAM_WITHOUT_DOWNSTREAM = 518,
+    VAR_FOR_UPSTREAM = 519,
+    VAR_AUTH_ZONE = 520,
+    VAR_ZONEFILE = 521,
+    VAR_MASTER = 522,
+    VAR_URL = 523,
+    VAR_FOR_DOWNSTREAM = 524,
+    VAR_FALLBACK_ENABLED = 525,
+    VAR_TLS_ADDITIONAL_PORT = 526,
+    VAR_LOW_RTT = 527,
+    VAR_LOW_RTT_PERMIL = 528,
+    VAR_FAST_SERVER_PERMIL = 529,
+    VAR_FAST_SERVER_NUM = 530,
+    VAR_ALLOW_NOTIFY = 531,
+    VAR_TLS_WIN_CERT = 532,
+    VAR_TCP_CONNECTION_LIMIT = 533,
+    VAR_FORWARD_NO_CACHE = 534,
+    VAR_STUB_NO_CACHE = 535,
+    VAR_LOG_SERVFAIL = 536,
+    VAR_DENY_ANY = 537,
+    VAR_UNKNOWN_SERVER_TIME_LIMIT = 538,
+    VAR_LOG_TAG_QUERYREPLY = 539,
+    VAR_STREAM_WAIT_SIZE = 540,
+    VAR_TLS_CIPHERS = 541,
+    VAR_TLS_CIPHERSUITES = 542,
+    VAR_TLS_USE_SNI = 543,
+    VAR_IPSET = 544,
+    VAR_IPSET_NAME_V4 = 545,
+    VAR_IPSET_NAME_V6 = 546,
+    VAR_TLS_SESSION_TICKET_KEYS = 547,
+    VAR_RPZ = 548,
+    VAR_TAGS = 549,
+    VAR_RPZ_ACTION_OVERRIDE = 550,
+    VAR_RPZ_CNAME_OVERRIDE = 551,
+    VAR_RPZ_LOG = 552,
+    VAR_RPZ_LOG_NAME = 553,
+    VAR_DYNLIB = 554,
+    VAR_DYNLIB_FILE = 555,
+    VAR_EDNS_CLIENT_STRING = 556,
+    VAR_EDNS_CLIENT_STRING_OPCODE = 557
   };
 #endif
 /* Tokens.  */
@@ -494,158 +497,161 @@ extern int yydebug;
 #define VAR_HTTP_QUERY_BUFFER_SIZE 400
 #define VAR_HTTP_RESPONSE_BUFFER_SIZE 401
 #define VAR_HTTP_NODELAY 402
-#define VAR_STUB_FIRST 403
-#define VAR_MINIMAL_RESPONSES 404
-#define VAR_RRSET_ROUNDROBIN 405
-#define VAR_MAX_UDP_SIZE 406
-#define VAR_DELAY_CLOSE 407
-#define VAR_UNBLOCK_LAN_ZONES 408
-#define VAR_INSECURE_LAN_ZONES 409
-#define VAR_INFRA_CACHE_MIN_RTT 410
-#define VAR_DNS64_PREFIX 411
-#define VAR_DNS64_SYNTHALL 412
-#define VAR_DNS64_IGNORE_AAAA 413
-#define VAR_DNSTAP 414
-#define VAR_DNSTAP_ENABLE 415
-#define VAR_DNSTAP_SOCKET_PATH 416
-#define VAR_DNSTAP_IP 417
-#define VAR_DNSTAP_TLS 418
-#define VAR_DNSTAP_TLS_SERVER_NAME 419
-#define VAR_DNSTAP_TLS_CERT_BUNDLE 420
-#define VAR_DNSTAP_TLS_CLIENT_KEY_FILE 421
-#define VAR_DNSTAP_TLS_CLIENT_CERT_FILE 422
-#define VAR_DNSTAP_SEND_IDENTITY 423
-#define VAR_DNSTAP_SEND_VERSION 424
-#define VAR_DNSTAP_BIDIRECTIONAL 425
-#define VAR_DNSTAP_IDENTITY 426
-#define VAR_DNSTAP_VERSION 427
-#define VAR_DNSTAP_LOG_RESOLVER_QUERY_MESSAGES 428
-#define VAR_DNSTAP_LOG_RESOLVER_RESPONSE_MESSAGES 429
-#define VAR_DNSTAP_LOG_CLIENT_QUERY_MESSAGES 430
-#define VAR_DNSTAP_LOG_CLIENT_RESPONSE_MESSAGES 431
-#define VAR_DNSTAP_LOG_FORWARDER_QUERY_MESSAGES 432
-#define VAR_DNSTAP_LOG_FORWARDER_RESPONSE_MESSAGES 433
-#define VAR_RESPONSE_IP_TAG 434
-#define VAR_RESPONSE_IP 435
-#define VAR_RESPONSE_IP_DATA 436
-#define VAR_HARDEN_ALGO_DOWNGRADE 437
-#define VAR_IP_TRANSPARENT 438
-#define VAR_IP_DSCP 439
-#define VAR_DISABLE_DNSSEC_LAME_CHECK 440
-#define VAR_IP_RATELIMIT 441
-#define VAR_IP_RATELIMIT_SLABS 442
-#define VAR_IP_RATELIMIT_SIZE 443
-#define VAR_RATELIMIT 444
-#define VAR_RATELIMIT_SLABS 445
-#define VAR_RATELIMIT_SIZE 446
-#define VAR_RATELIMIT_FOR_DOMAIN 447
-#define VAR_RATELIMIT_BELOW_DOMAIN 448
-#define VAR_IP_RATELIMIT_FACTOR 449
-#define VAR_RATELIMIT_FACTOR 450
-#define VAR_SEND_CLIENT_SUBNET 451
-#define VAR_CLIENT_SUBNET_ZONE 452
-#define VAR_CLIENT_SUBNET_ALWAYS_FORWARD 453
-#define VAR_CLIENT_SUBNET_OPCODE 454
-#define VAR_MAX_CLIENT_SUBNET_IPV4 455
-#define VAR_MAX_CLIENT_SUBNET_IPV6 456
-#define VAR_MIN_CLIENT_SUBNET_IPV4 457
-#define VAR_MIN_CLIENT_SUBNET_IPV6 458
-#define VAR_MAX_ECS_TREE_SIZE_IPV4 459
-#define VAR_MAX_ECS_TREE_SIZE_IPV6 460
-#define VAR_CAPS_WHITELIST 461
-#define VAR_CACHE_MAX_NEGATIVE_TTL 462
-#define VAR_PERMIT_SMALL_HOLDDOWN 463
-#define VAR_QNAME_MINIMISATION 464
-#define VAR_QNAME_MINIMISATION_STRICT 465
-#define VAR_IP_FREEBIND 466
-#define VAR_DEFINE_TAG 467
-#define VAR_LOCAL_ZONE_TAG 468
-#define VAR_ACCESS_CONTROL_TAG 469
-#define VAR_LOCAL_ZONE_OVERRIDE 470
-#define VAR_ACCESS_CONTROL_TAG_ACTION 471
-#define VAR_ACCESS_CONTROL_TAG_DATA 472
-#define VAR_VIEW 473
-#define VAR_ACCESS_CONTROL_VIEW 474
-#define VAR_VIEW_FIRST 475
-#define VAR_SERVE_EXPIRED 476
-#define VAR_SERVE_EXPIRED_TTL 477
-#define VAR_SERVE_EXPIRED_TTL_RESET 478
-#define VAR_SERVE_EXPIRED_REPLY_TTL 479
-#define VAR_SERVE_EXPIRED_CLIENT_TIMEOUT 480
-#define VAR_FAKE_DSA 481
-#define VAR_FAKE_SHA1 482
-#define VAR_LOG_IDENTITY 483
-#define VAR_HIDE_TRUSTANCHOR 484
-#define VAR_TRUST_ANCHOR_SIGNALING 485
-#define VAR_AGGRESSIVE_NSEC 486
-#define VAR_USE_SYSTEMD 487
-#define VAR_SHM_ENABLE 488
-#define VAR_SHM_KEY 489
-#define VAR_ROOT_KEY_SENTINEL 490
-#define VAR_DNSCRYPT 491
-#define VAR_DNSCRYPT_ENABLE 492
-#define VAR_DNSCRYPT_PORT 493
-#define VAR_DNSCRYPT_PROVIDER 494
-#define VAR_DNSCRYPT_SECRET_KEY 495
-#define VAR_DNSCRYPT_PROVIDER_CERT 496
-#define VAR_DNSCRYPT_PROVIDER_CERT_ROTATED 497
-#define VAR_DNSCRYPT_SHARED_SECRET_CACHE_SIZE 498
-#define VAR_DNSCRYPT_SHARED_SECRET_CACHE_SLABS 499
-#define VAR_DNSCRYPT_NONCE_CACHE_SIZE 500
-#define VAR_DNSCRYPT_NONCE_CACHE_SLABS 501
-#define VAR_IPSECMOD_ENABLED 502
-#define VAR_IPSECMOD_HOOK 503
-#define VAR_IPSECMOD_IGNORE_BOGUS 504
-#define VAR_IPSECMOD_MAX_TTL 505
-#define VAR_IPSECMOD_WHITELIST 506
-#define VAR_IPSECMOD_STRICT 507
-#define VAR_CACHEDB 508
-#define VAR_CACHEDB_BACKEND 509
-#define VAR_CACHEDB_SECRETSEED 510
-#define VAR_CACHEDB_REDISHOST 511
-#define VAR_CACHEDB_REDISPORT 512
-#define VAR_CACHEDB_REDISTIMEOUT 513
-#define VAR_CACHEDB_REDISEXPIRERECORDS 514
-#define VAR_UDP_UPSTREAM_WITHOUT_DOWNSTREAM 515
-#define VAR_FOR_UPSTREAM 516
-#define VAR_AUTH_ZONE 517
-#define VAR_ZONEFILE 518
-#define VAR_MASTER 519
-#define VAR_URL 520
-#define VAR_FOR_DOWNSTREAM 521
-#define VAR_FALLBACK_ENABLED 522
-#define VAR_TLS_ADDITIONAL_PORT 523
-#define VAR_LOW_RTT 524
-#define VAR_LOW_RTT_PERMIL 525
-#define VAR_FAST_SERVER_PERMIL 526
-#define VAR_FAST_SERVER_NUM 527
-#define VAR_ALLOW_NOTIFY 528
-#define VAR_TLS_WIN_CERT 529
-#define VAR_TCP_CONNECTION_LIMIT 530
-#define VAR_FORWARD_NO_CACHE 531
-#define VAR_STUB_NO_CACHE 532
-#define VAR_LOG_SERVFAIL 533
-#define VAR_DENY_ANY 534
-#define VAR_UNKNOWN_SERVER_TIME_LIMIT 535
-#define VAR_LOG_TAG_QUERYREPLY 536
-#define VAR_STREAM_WAIT_SIZE 537
-#define VAR_TLS_CIPHERS 538
-#define VAR_TLS_CIPHERSUITES 539
-#define VAR_TLS_USE_SNI 540
-#define VAR_IPSET 541
-#define VAR_IPSET_NAME_V4 542
-#define VAR_IPSET_NAME_V6 543
-#define VAR_TLS_SESSION_TICKET_KEYS 544
-#define VAR_RPZ 545
-#define VAR_TAGS 546
-#define VAR_RPZ_ACTION_OVERRIDE 547
-#define VAR_RPZ_CNAME_OVERRIDE 548
-#define VAR_RPZ_LOG 549
-#define VAR_RPZ_LOG_NAME 550
-#define VAR_DYNLIB 551
-#define VAR_DYNLIB_FILE 552
-#define VAR_EDNS_CLIENT_TAG 553
-#define VAR_EDNS_CLIENT_TAG_OPCODE 554
+#define VAR_HTTP_NOTLS_DOWNSTREAM 403
+#define VAR_STUB_FIRST 404
+#define VAR_MINIMAL_RESPONSES 405
+#define VAR_RRSET_ROUNDROBIN 406
+#define VAR_MAX_UDP_SIZE 407
+#define VAR_DELAY_CLOSE 408
+#define VAR_UDP_CONNECT 409
+#define VAR_UNBLOCK_LAN_ZONES 410
+#define VAR_INSECURE_LAN_ZONES 411
+#define VAR_INFRA_CACHE_MIN_RTT 412
+#define VAR_INFRA_KEEP_PROBING 413
+#define VAR_DNS64_PREFIX 414
+#define VAR_DNS64_SYNTHALL 415
+#define VAR_DNS64_IGNORE_AAAA 416
+#define VAR_DNSTAP 417
+#define VAR_DNSTAP_ENABLE 418
+#define VAR_DNSTAP_SOCKET_PATH 419
+#define VAR_DNSTAP_IP 420
+#define VAR_DNSTAP_TLS 421
+#define VAR_DNSTAP_TLS_SERVER_NAME 422
+#define VAR_DNSTAP_TLS_CERT_BUNDLE 423
+#define VAR_DNSTAP_TLS_CLIENT_KEY_FILE 424
+#define VAR_DNSTAP_TLS_CLIENT_CERT_FILE 425
+#define VAR_DNSTAP_SEND_IDENTITY 426
+#define VAR_DNSTAP_SEND_VERSION 427
+#define VAR_DNSTAP_BIDIRECTIONAL 428
+#define VAR_DNSTAP_IDENTITY 429
+#define VAR_DNSTAP_VERSION 430
+#define VAR_DNSTAP_LOG_RESOLVER_QUERY_MESSAGES 431
+#define VAR_DNSTAP_LOG_RESOLVER_RESPONSE_MESSAGES 432
+#define VAR_DNSTAP_LOG_CLIENT_QUERY_MESSAGES 433
+#define VAR_DNSTAP_LOG_CLIENT_RESPONSE_MESSAGES 434
+#define VAR_DNSTAP_LOG_FORWARDER_QUERY_MESSAGES 435
+#define VAR_DNSTAP_LOG_FORWARDER_RESPONSE_MESSAGES 436
+#define VAR_RESPONSE_IP_TAG 437
+#define VAR_RESPONSE_IP 438
+#define VAR_RESPONSE_IP_DATA 439
+#define VAR_HARDEN_ALGO_DOWNGRADE 440
+#define VAR_IP_TRANSPARENT 441
+#define VAR_IP_DSCP 442
+#define VAR_DISABLE_DNSSEC_LAME_CHECK 443
+#define VAR_IP_RATELIMIT 444
+#define VAR_IP_RATELIMIT_SLABS 445
+#define VAR_IP_RATELIMIT_SIZE 446
+#define VAR_RATELIMIT 447
+#define VAR_RATELIMIT_SLABS 448
+#define VAR_RATELIMIT_SIZE 449
+#define VAR_RATELIMIT_FOR_DOMAIN 450
+#define VAR_RATELIMIT_BELOW_DOMAIN 451
+#define VAR_IP_RATELIMIT_FACTOR 452
+#define VAR_RATELIMIT_FACTOR 453
+#define VAR_SEND_CLIENT_SUBNET 454
+#define VAR_CLIENT_SUBNET_ZONE 455
+#define VAR_CLIENT_SUBNET_ALWAYS_FORWARD 456
+#define VAR_CLIENT_SUBNET_OPCODE 457
+#define VAR_MAX_CLIENT_SUBNET_IPV4 458
+#define VAR_MAX_CLIENT_SUBNET_IPV6 459
+#define VAR_MIN_CLIENT_SUBNET_IPV4 460
+#define VAR_MIN_CLIENT_SUBNET_IPV6 461
+#define VAR_MAX_ECS_TREE_SIZE_IPV4 462
+#define VAR_MAX_ECS_TREE_SIZE_IPV6 463
+#define VAR_CAPS_WHITELIST 464
+#define VAR_CACHE_MAX_NEGATIVE_TTL 465
+#define VAR_PERMIT_SMALL_HOLDDOWN 466
+#define VAR_QNAME_MINIMISATION 467
+#define VAR_QNAME_MINIMISATION_STRICT 468
+#define VAR_IP_FREEBIND 469
+#define VAR_DEFINE_TAG 470
+#define VAR_LOCAL_ZONE_TAG 471
+#define VAR_ACCESS_CONTROL_TAG 472
+#define VAR_LOCAL_ZONE_OVERRIDE 473
+#define VAR_ACCESS_CONTROL_TAG_ACTION 474
+#define VAR_ACCESS_CONTROL_TAG_DATA 475
+#define VAR_VIEW 476
+#define VAR_ACCESS_CONTROL_VIEW 477
+#define VAR_VIEW_FIRST 478
+#define VAR_SERVE_EXPIRED 479
+#define VAR_SERVE_EXPIRED_TTL 480
+#define VAR_SERVE_EXPIRED_TTL_RESET 481
+#define VAR_SERVE_EXPIRED_REPLY_TTL 482
+#define VAR_SERVE_EXPIRED_CLIENT_TIMEOUT 483
+#define VAR_FAKE_DSA 484
+#define VAR_FAKE_SHA1 485
+#define VAR_LOG_IDENTITY 486
+#define VAR_HIDE_TRUSTANCHOR 487
+#define VAR_TRUST_ANCHOR_SIGNALING 488
+#define VAR_AGGRESSIVE_NSEC 489
+#define VAR_USE_SYSTEMD 490
+#define VAR_SHM_ENABLE 491
+#define VAR_SHM_KEY 492
+#define VAR_ROOT_KEY_SENTINEL 493
+#define VAR_DNSCRYPT 494
+#define VAR_DNSCRYPT_ENABLE 495
+#define VAR_DNSCRYPT_PORT 496
+#define VAR_DNSCRYPT_PROVIDER 497
+#define VAR_DNSCRYPT_SECRET_KEY 498
+#define VAR_DNSCRYPT_PROVIDER_CERT 499
+#define VAR_DNSCRYPT_PROVIDER_CERT_ROTATED 500
+#define VAR_DNSCRYPT_SHARED_SECRET_CACHE_SIZE 501
+#define VAR_DNSCRYPT_SHARED_SECRET_CACHE_SLABS 502
+#define VAR_DNSCRYPT_NONCE_CACHE_SIZE 503
+#define VAR_DNSCRYPT_NONCE_CACHE_SLABS 504
+#define VAR_IPSECMOD_ENABLED 505
+#define VAR_IPSECMOD_HOOK 506
+#define VAR_IPSECMOD_IGNORE_BOGUS 507
+#define VAR_IPSECMOD_MAX_TTL 508
+#define VAR_IPSECMOD_WHITELIST 509
+#define VAR_IPSECMOD_STRICT 510
+#define VAR_CACHEDB 511
+#define VAR_CACHEDB_BACKEND 512
+#define VAR_CACHEDB_SECRETSEED 513
+#define VAR_CACHEDB_REDISHOST 514
+#define VAR_CACHEDB_REDISPORT 515
+#define VAR_CACHEDB_REDISTIMEOUT 516
+#define VAR_CACHEDB_REDISEXPIRERECORDS 517
+#define VAR_UDP_UPSTREAM_WITHOUT_DOWNSTREAM 518
+#define VAR_FOR_UPSTREAM 519
+#define VAR_AUTH_ZONE 520
+#define VAR_ZONEFILE 521
+#define VAR_MASTER 522
+#define VAR_URL 523
+#define VAR_FOR_DOWNSTREAM 524
+#define VAR_FALLBACK_ENABLED 525
+#define VAR_TLS_ADDITIONAL_PORT 526
+#define VAR_LOW_RTT 527
+#define VAR_LOW_RTT_PERMIL 528
+#define VAR_FAST_SERVER_PERMIL 529
+#define VAR_FAST_SERVER_NUM 530
+#define VAR_ALLOW_NOTIFY 531
+#define VAR_TLS_WIN_CERT 532
+#define VAR_TCP_CONNECTION_LIMIT 533
+#define VAR_FORWARD_NO_CACHE 534
+#define VAR_STUB_NO_CACHE 535
+#define VAR_LOG_SERVFAIL 536
+#define VAR_DENY_ANY 537
+#define VAR_UNKNOWN_SERVER_TIME_LIMIT 538
+#define VAR_LOG_TAG_QUERYREPLY 539
+#define VAR_STREAM_WAIT_SIZE 540
+#define VAR_TLS_CIPHERS 541
+#define VAR_TLS_CIPHERSUITES 542
+#define VAR_TLS_USE_SNI 543
+#define VAR_IPSET 544
+#define VAR_IPSET_NAME_V4 545
+#define VAR_IPSET_NAME_V6 546
+#define VAR_TLS_SESSION_TICKET_KEYS 547
+#define VAR_RPZ 548
+#define VAR_TAGS 549
+#define VAR_RPZ_ACTION_OVERRIDE 550
+#define VAR_RPZ_CNAME_OVERRIDE 551
+#define VAR_RPZ_LOG 552
+#define VAR_RPZ_LOG_NAME 553
+#define VAR_DYNLIB 554
+#define VAR_DYNLIB_FILE 555
+#define VAR_EDNS_CLIENT_STRING 556
+#define VAR_EDNS_CLIENT_STRING_OPCODE 557
 
 /* Value type.  */
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
@@ -655,7 +661,7 @@ union YYSTYPE
 
 	char*	str;
 
-#line 659 "util/configparser.h"
+#line 665 "util/configparser.h"
 
 };
 typedef union YYSTYPE YYSTYPE;
